﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinListView;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;
using System.Threading;

namespace WLV_PreloadSomeImages
{
    class CustomCreationFilter :IUIElementCreationFilter
    {
        private List<UltraListViewItemUIElement> itemsInView;
        private List<UltraListViewItem> oldItemsInView;

        public CustomCreationFilter()
        {
            itemsInView = new List<UltraListViewItemUIElement>();
            oldItemsInView = new List<UltraListViewItem>();
        }

        public void AfterCreateChildElements(UIElement parent)
        {
            
            if (parent is UltraListViewItemContainerUIElement)
            {
                //get all items that have a UIElement, but no image
                itemsInView = parent.ChildElements.Where((x) => x is UltraListViewItemUIElement)
                     .Cast<UltraListViewItemUIElement>().Where((x)=>x.Item.Appearance.Image == null).ToList();

                //get all items that have an image, but no UIElement
                oldItemsInView = ((UltraListViewItemContainerUIElement)parent).ListView.Items.All.Cast<UltraListViewItem>()
                     .Where((x) => x.Appearance.Image != null && x.UIElement == null).ToList();

                if (oldItemsInView.Count == 0 && itemsInView.Count == 0)
                {
                    //if both lists are empty, there is no change in the displayed elements
                    return;
                }

                foreach (UltraListViewItemUIElement item in itemsInView)
                {
                    if ( item.Item.Appearance.Image == null )
                    {
                        item.Item.Appearance.Image = Image.FromFile(item.Item.Tag.ToString());
                    }
                }
                foreach (UltraListViewItem item in oldItemsInView)
                {
                    if (item.Appearance.Image != null)
                    {
                        ((Image)item.Appearance.Image).Dispose();
                        item.Appearance.Image = null;
                    }
                }
            }
        }
        
        public bool BeforeCreateChildElements(UIElement parent)
        {
            return false;
        }
    }
}
